//
// Report Service
//
// This service provides a dummy document definition for the purpose of this sample.  In 
// my real world usage, I split documentDef creation from the reportPDF creation.  The RptBuilderSvc
// is used to receive inputs and return a JSON object w/ the report declarations.  This mock svc
// just creates some random progress matrix and draws a table to display.  The pdfMake.org site
// has a nice playground for drafting your report pieces.  My plan is to share the ionic-pdf
// so users can incorporate PDF generation and focus on creating their docDefs and using ionic-pdf
// to easily render
(function() {
	'use strict';
	// attach the factories and service to the [starter.services] module in
	// angular
	angular.module('starter.services').service('ReportBuilderSvc',
			reportBuilderService);

	function reportBuilderService() {
		var self = this;

		self.generateReport = _generateReport;
		function _generateReport(daten) {
			// create an array of progress for the (6) categories presented
			// plz see the pdfMake.org site for examples of document definitions
			var user = daten.user;
			var contract = daten.contract;
			var placeDate = daten.placeDate ? daten.placeDate : "";
			var resiliateDate = daten.resiliateDate ? daten.resiliateDate : "";
			var street = user.street ? user.street : "";
			var citycode=user.citycode?user.citycode:'';
			var city = user.city ? user.city : "";
			var clientnumber=contract.clientnumber?contract.clientnumber:"";
			city=citycode + " "+city;
			var partnercity=contract.partnerCityCode?contract.partnerCityCode+" ":"";
			partnercity=partnercity + contract.partnerCity;
			user.country = user.country ? user.country : "";
			var lang = daten.lang ? daten.lang : "de";
			if ("de" == lang) {
				return {
					pageSize : 'A4',

					// by default we use portrait, you can change it to
					// landscape if
					// you wish
					pageOrientation : 'portrait',

					// [left, top, right, bottom] or [horizontal, vertical] or
					// just
					// a number for equal margins
					pageMargins : [ 40, 60, 40, 60 ],
					content : [
							{
								columns : [
										{
											stack : [ user.fullname,
													street, city,
													user.country, ' ', ' ',
													' ', contract.partner,
													contract.partnerStreet,
													partnercity,
													contract.partnerCountry ],
											width : 300
										}, {
											stack : [ placeDate, '' ],
											width : 300
										} ]
							},
							' ',
							' ',
							{
								text : 'Kundennummer: ' + clientnumber,
								fontSize : 15,
								bold : true
							},
							{
								text : 'Vertragsreferenz: ' + contract.referenz,
								fontSize : 15,
								bold : true
							},
							' ',
							{
								text : 'Kündigung',
								fontSize : 15,
								bold : true
							},
							
							{
								text : 'Sehr geehrte Damen und Herren,',
								margin : [ 0, 10, 0, 0 ]
							},
							{
								text : 'hiermit kündige ich meinen mit Ihnen abgeschlossenen Vertrag fristgerecht zum '
										+ resiliateDate
										+ ' bzw. zum nächstmöglichen Zeitpunkt.',
								margin : [ 0, 10, 0, 0 ]
							},
							{
								text : 'Bitte bestätigen Sie die Kündigung und den Beendigungstermin des Vertrags schriftlich.',
								margin : [ 0, 10, 0, 0 ]
							}, {
								text : 'Vielen Dank.',
								margin : [ 0, 10, 0, 0 ]
							}, {
								text : 'Mit freundlichen Grüßen',
								margin : [ 0, 10, 0, 0 ]
							}, {
								text : user.fullname,
								margin : [ 0, 20, 0, 0 ]
							} ]
				};
			}
			
			if ("fr" == lang) {
				return {
					pageSize : 'A4',

					// by default we use portrait, you can change it to
					// landscape if
					// you wish
					pageOrientation : 'portrait',

					// [left, top, right, bottom] or [horizontal, vertical] or
					// just
					// a number for equal margins
					pageMargins : [ 40, 60, 40, 60 ],
					content : [
							{
								columns : [
										{
											stack : [ user.fullname,
													street, city,
													user.country, ' ', ' ',
													' ', contract.partner,
													contract.partnerStreet,
													partnercity,
													contract.partnerCountry ],
											width : 300
										}, {
											stack : [ placeDate, '' ],
											width : 300
										} ]
							},
							' ',
							' ',
							{
								text : 'Numéro d\'identification: ' + clientnumber,
								fontSize : 15,
								bold : true
							},
							{
								text : 'Référence du contrat: ' + contract.referenz,
								fontSize : 15,
								bold : true
							},
							' ',
							{
								text : 'Résiliation',
								fontSize : 15,
								bold : true
							},
							
							{
								text : 'Très chèrs Mesdames et Messieurs ,',
								margin : [ 0, 10, 0, 0 ]
							},
							{
								text : 'Je viens par ce courrier mettre fin à mon contrat avec votre société à partir du  '
										+ resiliateDate
										,
								margin : [ 0, 10, 0, 0 ]
							},
							{
								text : 'Je vous pries de bien vouloir me confirmer la résiliation par écrit aussitôt que vous recevez ce message.',
								margin : [ 0, 10, 0, 0 ]
							}, {
								text : 'Je vous remercie',
								margin : [ 0, 10, 0, 0 ]
							}, {
								text : 'Recevez mes saltutations très distinguées.',
								margin : [ 0, 10, 0, 0 ]
							}, {
								text : user.fullname,
								margin : [ 0, 20, 0, 0 ]
							} ]
				};
			}

			
			if ("en" == lang) {
				return {
					pageSize : 'A4',

					// by default we use portrait, you can change it to
					// landscape if
					// you wish
					pageOrientation : 'portrait',

					// [left, top, right, bottom] or [horizontal, vertical] or
					// just
					// a number for equal margins
					pageMargins : [ 40, 60, 40, 60 ],
					content : [
							{
								columns : [
										{
											stack : [ user.fullname,
												     street, city,
													user.country, ' ', ' ',
													' ', contract.partner,
													contract.partnerStreet,
													partnercity,
													contract.partnerCountry ],
											width : 300
										}, {
											stack : [ placeDate, '' ],
											width : 300
										} ]
							},
							' ',
							' ',
							{
								text : 'Identificationnumber: ' + clientnumber,
								fontSize : 15,
								bold : true
							},
							{
								text : 'Contract reference: ' + contract.referenz,
								fontSize : 15,
								bold : true
							},
							' ',
							{
								text : 'Contract resiliation',
								fontSize : 15,
								bold : true
							},
							
							{
								text : 'Dear Sir and Dear madam,',
								margin : [ 0, 10, 0, 0 ]
							},
							{
								text : 'I am writing you today to cancel my contract with your company at '
										+ resiliateDate,
								margin : [ 0, 10, 0, 0 ]
							},
							{
								text : 'Please confirm me the cancelation as soon as you get this letter.',
								margin : [ 0, 10, 0, 0 ]
							}, {
								text : 'Tank you.',
								margin : [ 0, 10, 0, 0 ]
							}, {
								text : 'Sincerely yours.',
								margin : [ 0, 10, 0, 0 ]
							}, {
								text : user.fullname,
								margin : [ 0, 20, 0, 0 ]
							} ]
				};
			}
			
			return null;
		}
		;
	}
})();